require 'uu/os/server/server_initializer'
require 'uu/os/util/class_scanner'
require 'uu/os/controller/abstract_controller'

module UU
  module OS
    module Controller

      # Implementation of server startup listener for transparent
      # loading of all valid controllers available on load path.
      # As valid controller is considered any class inherited from
      # {UU::OS::Controller::AbstractController} with defined constant
      # +CONTROLLER_URL_PATH+.
      class ControllerLoader

        # Loads all available controllers on load path. As valid controller is considered
        # any class inherited from {UU::OS::Controller::AbstractController} with defined
        # constant +CONTROLLER_URL_PATH+.
        # @param [UU::OS::Server::ServerContext] ctx Server context
        # @return [void]
        def self.on_server_startup(ctx)
          controllers = UU::OS::Util::ClassScanner.find_classes do |clazz|
            clazz < UU::OS::Controller::AbstractController && clazz.const_defined?(:CONTROLLER_URL_PATH)
          end
          controllers.each do |controller|
            ctx.register_servlet(controller, controller::CONTROLLER_URL_PATH)
          end
        end

      end

    end
  end
end
